/* -*- Mode: C++; c-basic-offset: 4; indent-tabs-mode: nil; tab-width: 4 -*- */
/* vi: set ts=4 sw=4 expandtab: (add to ~/.vimrc: set modeline modelines=5) */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */


#ifndef __nanojit_NativeInterpreter__
#define __nanojit_NativeInterpreter__



#include "NativeCommon.h"

namespace nanojit
{

typedef uint8_t NIns;

/* ARM registers */
static const Register
	R0  = { 0 },
	R1  = { 1 },
	R2  = { 2 },
	R3  = { 3 },
	R4  = { 4 },
	R5  = { 5 },
	R6  = { 6 },
	R7  = { 7 },
	R8  = { 8 },
	R9  = { 9 },
	R10 = { 10 },
	R11 = { 11 },
	R12 = { 12 },
	R13 = { 13 },
	R14 = { 14 },
	R15 = { 15 },
	R16 = { 16 },
	R17 = { 17 },
	R18 = { 18 },
	R19 = { 19 },
	R20 = { 20 },
	R21 = { 21 },
	R22 = { 22 },
	R23 = { 23 },
	R24 = { 24 },
	R25 = { 25 },
	R26 = { 26 },
	R27 = { 27 },
	R28 = { 28 },
	R29 = { 29 },
	R30 = { 30 },
	R31 = { 31 },
	FP = R31, // excluded from managed register list
	UnspecifiedReg = { 128 };
	


static const uint32_t FirstRegNum = 0; /* R0 */
static const uint32_t LastRegNum = 31; /* R31 */ 

typedef uint32_t RegisterMask;

static const RegisterMask GpRegs = 0xffffffff;
static const RegisterMask FpRegs = 0xffffffff;

verbose_only( extern const char* regNames[]; )

#define DECLARE_PLATFORM_REGALLOC() \
	void _platform_regalloc() const

#define DECLARE_PLATFORM_ASSEMBLER_DEBUG()

#define DECLARE_PLATFORM_ASSEMBLER()                                                                              \
	static const unsigned int _nCodeBlockSizeThresholdInBytes = 0;                                                \
	inline void nNotifyPageJump() { }                                                                             \
	bool hardenNopInsertion(const Config& /*c*/) { return false; }                                                \
	void arm_arith_gen_flags(LIns* ins, Register rr, Register ra, Register rb);                                   \
	uint64_t createFunctionSignature(LIns* callIns, const ParameterValue* callParams, unsigned int paramCount);   \
	void asm_restore_remat(LIns *ins, Register r);                                                                \
	void nImmI(Register r, uint32_t v);                                                                        \
	void nImmQ(Register r, uint64_t v);                                                                        \
	void nImmF(Register r, float v);                                                                           \
	void nImmD(Register r, double v);                                                                          \
	void asm_op1(LIns* ins);                                                                                      \
	void asm_op2(LIns* ins);                                                                                      \
	void asm_force_gen_flags(LIns* ins);                                                                          \


#define NANOJIT_RA_PREFERS_LSREG

#define NANOJIT_MAX_STACK_ENTRY 1024

#define NANOJIT_SMALLEST_UNDERRUN_PROT 1


} // namespace nanojit
#endif // __nanojit_NativeARM__
